CLINSIG_MAP = {
    0: 'Uncertain significance',
    1: 'not provided',
    2: 'Benign',
    3: 'Likely benign',
    4: 'Likely pathogenic',
    5: 'Pathogenic',
    6: 'drug response',
    7: 'histocompatibility',
    255: 'other'
}
REV_CLINSIG_MAP = {
    'uncertain_significance': 0,
    'not_provided': 1,
    'benign': 2,
    'likely_benign': 3,
    'likely_pathogenic': 4,
    'pathogenic': 5,
    'drug_response': 6,
    'histocompatibility': 7,
    'other': 255
}
