PHENOTYPE_GROUPS = {
    'HP:0001298': {
        'name': 'Encephalopathy',
        'abbr': 'ENC'
    },
    'HP:0012759': {
        'name': 'Neurodevelopmental abnormality',
        'abbr': 'NDEV'
    },
    'HP:0001250': {
        'name': 'Seizures',
        'abbr': 'EP'
    },
    'HP:0100022': {
        'name': 'Abnormality of movement',
        'abbr': 'MOVE'
    },
    'HP:0000707': {
        'name': 'Neurology, other',
        'abbr': 'NEUR'
    },
    'HP:0003011': {
        'name': 'Abnormality of the musculature',
        'abbr': 'MUSC'
    },
    'HP:0001638': {
        'name': 'Cardiomyopathy',
        'abbr': 'CARD'
    },
    'HP:0001507': {
        'name': 'Growth abnormality',
        'abbr': 'GROW'
    },
    'HP:0001392': {
        'name': 'Abnormality of the liver',
        'abbr': 'LIV'
    },
    'HP:0011458': {
        'name': 'Abdominal symptom',
        'abbr': 'GI'
    },
    'HP:0012373': {
        'name': 'Abnormal eye physiology',
        'abbr': 'EYE'
    },
    'HP:0000077': {
        'name': 'Abnormality of the kidney',
        'abbr': 'KIDN'
    },
    'HP:0000951': {
        'name': 'Abnormality of the skin',
        'abbr': 'SKIN'
    },
    'HP:0001939': {
        'name': 'Abnormality of metabolism/homeostasis',
        'abbr': 'METAB'
    },
    'HP:0000118': {
        'name': 'As yet undefined/to be added',
        'abbr': 'UND'
    },
    'HP:0002011': {
        'name': 'Abnormal CNS morphology',
        'abbr': 'MRI'
    }
}

COHORT_TAGS = [
    'endo',
    'mito',
    'ketogenic diet',
    'pedhep',
    'other',
]